#!/bin/bash

OK='\033[32m'OK'\033[0m'
INFO='\033[32m'INFO'\033[0m'
FAILED='\033[31m'FAILED'\033[0m'
WARNING='\033[33m'WARNING'\033[0m'

WARNING_COUNT=0
FAILED_COUNT=0

MOUNT_PATH='/mnt/IVS/'
UI_PATH=''
BASE_PATH=''
PYTHON=''

RESPONSE_EXTEND=''


#######################################
# Show message with yellow color
# ARGUMENTS:
#   $1: task name
#   $2: function
#   $3: arguments
# OUTPUTS:
#   Logs the result of function.
#######################################
echo_title_message() {
    echo -e '\033[36m'$@'\033[0m'
}

#######################################
# Wrap function. It executes the command or bash function with arguments. And logs the result of function.
# It checks the command exit status or function exit status(via return keyword)
#
# ARGUMENTS:
#   $1: task name
#   $2: function
#   $3: arguments
# OUTPUTS:
#   Logs the result of function.
#######################################
execute_checking_function() {
    local task=$1
    local func=$2
    shift 2

    RESPONSE_EXTEND=''

    $func "$@"

    if [ $? -eq 0 ]; then
        echo -e "[${OK}] ${task} ${RESPONSE_EXTEND}"
    else
        echo -e "[${FAILED}] ${task} ${RESPONSE_EXTEND}"
        ((FAILED_COUNT=FAILED_COUNT+1))
    fi
}

check_path_exists() {
    local path=$1
    shift 1
    if [ -f "${path}" ]; then
        return 1
    else
        return 0
    fi
}

get_ui_service_path() {
    input_string=$(systemctl show -p ExecStart ivs-ui.service)
    if [[ $input_string =~ argv\[\]=/bin/bash\ (/[^/]*/[^/]*/[^/]*)/ ]]; then
        UI_PATH="${BASH_REMATCH[1]}"
        return 0
    else
        RESPONSE_EXTEND='Get service info (ivs-ui path) fail'
        return 1
    fi
}

get_base_service_path() {
    input_string=$(systemctl show -p ExecStart ivs-base.service)
    result=$(echo $input_string | grep -oP '/\K[^ ]*' | grep -oP '.*(?=/qvsd.sh)')
    if ! [ -n "$results" ]; then
        BASE_PATH="${result}"
        return 0
    else
        RESPONSE_EXTEND='Get service info (ivs-base path) fail'
        return 1
    fi
}

get_python_service_path() {
    input_string=$(systemctl show -p ExecStart ivs-python.service)
    result=$(echo $input_string | grep -oP '/\K[^ ]*' | grep -oP '.*(?=/qvsd.sh)')
    if ! [ -n "$results" ]; then
        PYTHON="${result}/usr/bin/python3"
        export LD_LIBRARY_PATH="${BASE_PATH}/usr/lib:${result}/usr/lib"
        return 0
    else
        RESPONSE_EXTEND='Get service info (ivs-python path) fail'
        return 1
    fi
}

human_readable() {
    local -i bytes=$1
    if (( bytes < 1024 )); then
        echo "${bytes}B"
    elif (( bytes < 1048576 )); then
        echo "$(( (bytes + 512) / 1024 ))KB"
    elif (( bytes < 1073741824 )); then
        echo "$(( (bytes + 524288) / 1048576 ))MB"
    else
        echo "$(( (bytes + 536870912) / 1073741824 ))GB"
    fi
}

#######################################
# Check the file which is existed or not.
# ARGUMENTS:
#   $1: command
#######################################
check_is_file_existed() {
    local file=$1
    if [ -e ${file} ]; then
        return 0
    else
        return 1
    fi
}

#######################################
# Check the directory which is existed or not.
# ARGUMENTS:
#   $1: command
#######################################
check_is_dic_existed() {
    local dic=$1
    if [ -d ${dic} ]; then
        return 0
    else
        return 1
    fi
}


#######################################
# Is service active or not
# ARGUMENTS:
#   $1: command
#######################################
systemctl_is_service_active() {
    local service_name=$1
    systemctl is-active ${service_name} > /dev/null 2>&1
}

#######################################
# Description   : VM disk image missing with disk I/O issue
# Mantis        :
# Jira          : IEIIVSTA-470
# ARGUMENTS:
#
# RETURN:
#   command exit status
#######################################
jira_ieiivsta_470_diskio_test() {
    local ivs_fs=$(df -P ${MOUNT_PATH} | tail -n 1 | awk '{print $1}')
    local ui_fs=$(df -P ${UI_PATH} | tail -n 1 | awk '{print $1}')

    echo "Test ${MOUNT_PATH} (${ivs_fs}) disk I/O"
    sudo dd if=$ivs_fs of=/dev/null bs=1G count=4

    if [ "$ivs_fs" != "$ui_fs" ]; then
        echo "Test ${UI_PATH} (${ui_fs}) disk I/O"
        sudo dd if=$ui_fs of=/dev/null bs=1G count=4
    else
        echo "${UI_PATH} and ${MOUNT_PATH} at the same filesystem (${ui_fs})"
    fi

    return 0
}

#######################################
# Description   : Disk remaind space not enough
# Mantis        :
# Jira          : Maintis 13566, 78301, 80127, 93216, 102794
# ARGUMENTS:
#
# RETURN:
#   command exit status
#######################################
maintis_13566_check_remaind_space() {
    local service_fs=$(df -P ${UI_PATH} | tail -n 1 | awk '{print $1}')
    local mount_fs=$(df -P ${MOUNT_PATH} | tail -n 1 | awk '{print $1}')

    local mount_remaind_space=$(df ${MOUNT_PATH} | tail -n 1 | awk '{print $4}')
    local service_remaind_space=$(df ${UI_PATH} | tail -n 1 | awk '{print $4}')
    local minimum_required=20971520
    echo -e "${MOUNT_PATH} (${mount_fs}) Remaining Space: $(human_readable mount_remaind_space*1024)"
    echo -e "${UI_PATH} (${service_fs}) Remaining Space: $(human_readable service_remaind_space*1024)"

    if [ $mount_remaind_space -gt $minimum_required ] && [ $service_remaind_space -gt $minimum_required ]; then
        return 0
    else
        RESPONSE_EXTEND='Remaining Space is less than 20GB'
        return 1
    fi
}

#######################################
# Description   : Database crash
# Mantis        :
# Jira          : Maintis 79476, 85265, 85420, 86004, 92164, 93017
# ARGUMENTS:
#
# RETURN:
#   command exit status
#######################################
maintis_79476_db_check() {
    local db_file="${UI_PATH}/vs.db"
    result=$(${PYTHON} -c "
import sqlite3
conn = sqlite3.connect('$db_file')
cursor = conn.cursor()
cursor.execute('PRAGMA integrity_check;')
print(cursor.fetchone()[0])
")
    if [[ "$result" == "ok" ]]; then
        return 0
    else
        echo -e "[${FAILED}] Database crash"
        return 1
    fi
}

#######################################
# Description   : iVEC file missing / damaged
# Mantis        :
# Jira          : Maintis 84892, 80814, 81558, 81812, 83232, 88365, 91624
# ARGUMENTS:
#
# RETURN:
#   command exit status
#######################################
maintis_84892_fs_check() {
    local dumpe2fs=$(which dumpe2fs)
    local mnt_fs=$(df -P ${MOUNT_PATH} | tail -n 1 | awk '{print $1}')
    local ui_fs=$(df -P ${UI_PATH} | tail -n 1 | awk '{print $1}')

    local res=$(sudo $dumpe2fs -h $mnt_fs 2>&1 | grep -e 'clean' -e 'Clean')
    if [[ -z "$res" ]]; then
        RESPONSE_EXTEND="${MOUNT_PATH}(${mnt_fs}) file system status error"
        return 1
    fi

    res=$(sudo $dumpe2fs -h $ui_fs 2>&1 | grep -e 'clean' -e 'Clean')
    if [[ -z "$res" ]]; then
        RESPONSE_EXTEND="${UI_PATH}(${ui_fs}) file system status error"
        return 1
    fi

    return 0
}

#######################################
# Description   : VM disk image broken
# Mantis        :
# Jira          : Maintis 102733, 103330
# ARGUMENTS:
#
# RETURN:
#   command exit status
#######################################
maintis_102733_vm_images_check() {
    local virsh="${BASE_PATH}/usr/bin/virsh"
    local qemu_img="${BASE_PATH}/usr/bin/qemu-img"
    local vms=$(sudo $virsh list --all --name)

    for vm in $vms; do
        vm_config=$(sudo $virsh dumpxml $vm)
        vm_state=$(sudo $virsh domstate $vm)
        disk_image_paths=$(echo "$vm_config" | grep 'source file' | awk -F\' '{print $2}')

        # 對每一個磁碟映像檔路徑進行迴圈
        for disk_image_path in $disk_image_paths; do
            disk_image_corrupt=$($qemu_img info -U $disk_image_path | grep 'corrupt:' | awk '{print $2}')
            if [ "$disk_image_corrupt" == "true" ]; then
                if [ -n "$RESPONSE_EXTEND" ]; then
                    RESPONSE_EXTEND="${RESPONSE_EXTEND}, ${disk_image_path}"
                else
                    RESPONSE_EXTEND="Broken image: ${disk_image_path}"
                fi
            fi
        done
    done

    if [ -n "$RESPONSE_EXTEND" ]; then
        return 1
    else
        return 0
    fi
}

#######################################
# Description   : VM xml file missing (.meta)
# Mantis        :
# Jira          : Maintis 98670, 102034
# ARGUMENTS:
#
# RETURN:
#   command exit status
#######################################
maintis_98670_vm_xml_file_check() {
    local virsh="${BASE_PATH}/usr/bin/virsh"
    local vms=$($virsh list --all --name)

    for vm in $vms; do
        vm_config=$($virsh dumpxml $vm)
        meta_paths=$(echo "$vm_config" | grep 'meta_path' | awk -F\> '{print $2}'| awk -F\< '{print $1}')
        if check_is_dic_existed meta_paths; then
            RESPONSE_EXTEND="Folder missing ${meta_paths}"
            return 1
        fi
        if ! [[ -f "$meta_paths/$vm.xml" ]]; then
            RESPONSE_EXTEND="File missing: ${meta_paths}/${vm}.xml"
            return 1
        fi
    done

    return 0
}

#######################################
# Use brctl to check iVEC bridge if existing
# ARGUMENTS:
#######################################
check_bridge() {
    br_name="ieibr"
    bridges=$(brctl show | grep $br_name | awk '{print $1}'| paste -sd ",")
    if [ x"${bridges}" != x"" ]; then
        RESPONSE_EXTEND=$bridges
        return 0
    else
        RESPONSE_EXTEND="No ieibrX"
        return 1
    fi
}

#######################################
# Execute the command. It returns executing result $? to caller
# ARGUMENTS:
#   $1: command
#######################################
check_cmd_executing() {
    local cmd=$1
    shift 1
    ${cmd} "$@" > /dev/null 2>&1
}

#######################################
# Show basic system information
# ARGUMENTS:
#######################################
system_info() {
    echo_title_message "### system information"

    local arch=$(uname -m)
    local kernel_version=$(uname -r)
    local platform=$(cat /etc/lsb-release | grep DESCRIPTION | cut -d'"' -f2)

    echo -e "PLATFORM: \t$platform"
    echo -e "ARCH: \t\t$arch"
    echo -e "KERNEL VERSION: $kernel_version"

    sudo dmidecode | grep -A3 '^System Information'

    echo -e "Memory Info:"
    egrep 'Mem|Cache|Buffers|Swap' /proc/meminfo | sed 's/^/  /'

    echo -e 'Network Info:'
    ip addr | sed -r ':a;N;$!ba;s/\n\s/ /g' | sed -r -n -e 's/^([0-9]+):\s(\w+).*(link\/(\w+))\s[a-f0-9:.]{,17}\sbrd\s[a-f0-9:.]{,17}\s*(inet\s([0-9]{1,3}(\.[0-9]{1,3}){3})).*/IFACE \2 \6 \4/p' -e 's/^([0-9]+):\s(\w+).*(link\/(\w+))\s[a-f0-9:.]{,17}\sbrd\s[a-f0-9:.]{,17}.*/IFACE \2 0.0.0.0 \4/p'

    echo -e ""
}

dmesg_info() {
    echo_title_message "### lookup dmesg error log"

    sudo dmesg | grep -i 'error\|fail\|panic'

    echo -e ""
}

checking_env() {
    echo_title_message "### checking env"

    execute_checking_function "checking ieibrX bridge: " check_cmd_executing check_bridge
    execute_checking_function "checking mount folder(${MOUNT_PATH})" check_is_dic_existed "${MOUNT_PATH}"

    echo -e ""
}

checking_service() {
    echo_title_message "### checking service"
    execute_checking_function "checking nginx.service is active (running)" check_cmd_executing systemctl_is_service_active nginx.service

    services=($(systemctl list-unit-files --type service | grep -E "^ivs-" | cut -d' ' -f1))
    for service in "${services[@]}"
    do
        execute_checking_function "checking ${service} is active (running)" check_cmd_executing systemctl_is_service_active ${service}
    done

    echo -e ""
}

checking_issue() {
    echo_title_message "### checking issue"

    execute_checking_function "Check system remaind space." maintis_13566_check_remaind_space
    execute_checking_function "Check datebase." maintis_79476_db_check
    execute_checking_function "Check file system status." maintis_84892_fs_check
    execute_checking_function "Test ${MOUNT_PATH} disk IO." jira_ieiivsta_470_diskio_test
    execute_checking_function "Check vm meta xml file if exists." maintis_98670_vm_xml_file_check
    execute_checking_function "Check all vm's disk image status." maintis_102733_vm_images_check

    echo -e ""
}

get_service_info() {
    get_ui_service_path
    get_base_service_path
    get_python_service_path
}

print_ivec_log_path() {
    echo_title_message "### iVEC logs"
    echo -e "Workers log directory:\t${UI_PATH}/var/log/workers/"
    echo -e "uWSGI log:\t\t${UI_PATH}/var/log/uwsgi.log"
    echo -e "QEMU log directory:\t${BASE_PATH}/var/log/libvirt/qemu/"
    echo -e "Libvirt log:\t\t${BASE_PATH}/var/log/libvirt/libvirtd.log"
    echo -e "Service script log:\t${UI_PATH}/var/log/qvsd.log or ${MOUNT_PATH}.ivs-ui/Log/qvsd.log"
    echo -e "System Log:\t\t${UI_PATH}/var/log/system.log or ${MOUNT_PATH}.ivs-ui/Log/system.log"
    echo -e "Manage Log:\t\t${UI_PATH}/var/log/qvs-manage.log or ${MOUNT_PATH}.ivs-ui/Log/qvs-manage.log"
    echo -e ""
}


main() {
    get_service_info

    system_info
    dmesg_info
    checking_env
    checking_service
    checking_issue

    print_ivec_log_path
}

main $1
echo -e "\n[${INFO}] Warning:${WARNING_COUNT}, Failed:${FAILED_COUNT}"

